// LICENSE : MIT
"use strict";
// Logger Context object

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _SourceNode = require("./nodes/SourceNode");

var _SourceNode2 = _interopRequireDefault(_SourceNode);

var _LoggerNodeGraph = require("./LoggerNodeGraph");

var _LoggerNodeGraph2 = _interopRequireDefault(_LoggerNodeGraph);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LoggerContext = function () {
    function LoggerContext(logQueue) {
        _classCallCheck(this, LoggerContext);

        this._logQueue = logQueue;
        this._sourceNodes = [];
        this._nodeGraph = new _LoggerNodeGraph2.default();
    }

    /**
     * create SourceNode and register it as input nodes.
     * @returns {SourceNode}
     */


    _createClass(LoggerContext, [{
        key: "createSourceNode",
        value: function createSourceNode() {
            var sourceNode = new _SourceNode2.default();
            this.addSourceNode(sourceNode);
            return sourceNode;
        }

        /**
         * add `sourceNode` as Source Node.
         * Source means that it have not parent node.
         * Well-known Input Source Node.
         * @param {SourceNode} sourceNode
         */

    }, {
        key: "addSourceNode",
        value: function addSourceNode(sourceNode) {
            this._sourceNodes.push(sourceNode);
        }

        /**
         * consume all logs from logQueue and process each log.
         */

    }, {
        key: "process",
        value: function process() {
            var _this = this;

            var logs = this._logQueue.consumeLogs();
            logs.forEach(function (logChunk) {
                _this._sourceNodes.forEach(function (sourceNode) {
                    sourceNode.processIfNecessary(logChunk);
                });
            });
        }

        /**
         * create Node dependencies graph from exist sourceNodes.
         * @param {SourceNode[]} [sourceNodes]
         * @returns {LoggerNodeGraph}
         */

    }, {
        key: "createNodeGraph",
        value: function createNodeGraph() {
            var sourceNodes = arguments.length <= 0 || arguments[0] === undefined ? this._sourceNodes : arguments[0];

            return this._nodeGraph.build(sourceNodes);
        }
    }]);

    return LoggerContext;
}();

exports.default = LoggerContext;
//# sourceMappingURL=LoggerContext.js.map