// LICENSE : MIT
"use strict";
/**
 * @typedef {Object} LeafNode
 * @property {string} type
 * @property {LoggerNode} node
 * @property {LeafNode[]} children
 */

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _walker = require("./utils/walker");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Create Logger Node Dependencies Graph and traverse it
 */

var LoggerNodeGraph = function () {
    function LoggerNodeGraph() {
        _classCallCheck(this, LoggerNodeGraph);

        this.root = this.createRootNode();
    }

    /**
     * create root node
     * @returns {{type: string, children: Array}}
     */


    _createClass(LoggerNodeGraph, [{
        key: "createRootNode",
        value: function createRootNode() {
            return {
                type: "root",
                children: []
            };
        }

        /**
         *
         * @param {LoggerNode} node
         * @param {LeafNode} LeafNode
         */

    }, {
        key: "buildChild",
        value: function buildChild(node, LeafNode) {
            var _this = this;

            var outputs = node.outputs;
            if (!outputs) {
                return;
            }
            var leafNode = this.addChild(node, LeafNode);
            outputs.forEach(function (childOutput) {
                _this.buildChild(childOutput, leafNode);
            });
        }

        /**
         * @param {LoggerNode[]} sourceNodes
         * @returns {LoggerNodeGraph}
         */

    }, {
        key: "build",
        value: function build(sourceNodes) {
            var _this2 = this;

            this.clear();
            sourceNodes.forEach(function (child) {
                _this2.buildChild(child, _this2.root);
            });
            return this;
        }
    }, {
        key: "traverse",
        value: function traverse(_ref) {
            var enter = _ref.enter;
            var leave = _ref.leave;

            return (0, _walker.walk)(this.root, { enter: enter, leave: leave });
        }

        /**
         *
         * @param {LoggerNode} node
         * @param {LeafNode} parentLeafNode
         * @returns {LeafNode}
         */

    }, {
        key: "addChild",
        value: function addChild(node, parentLeafNode) {
            var leafNode = {};
            Object.defineProperties(leafNode, {
                type: {
                    value: node.constructor.name || "<TODO>",
                    enumerable: true
                },
                node: {
                    value: node,
                    enumerable: false
                },
                children: {
                    value: [],
                    enumerable: true
                }
            });
            parentLeafNode.children.push(leafNode);
            return leafNode;
        }

        /**
         * reset current node tree
         */

    }, {
        key: "clear",
        value: function clear() {
            this.root = null;
            this.root = this.createRootNode();
        }
    }]);

    return LoggerNodeGraph;
}();

exports.default = LoggerNodeGraph;
//# sourceMappingURL=LoggerNodeGraph.js.map