// LICENSE : MIT
"use strict";
/**
 * LoggerNode is abstract Node interface.
 * LoggerNode connect next LoggerNode.
 * It is called "pipeline" mechanism.
 * @example
 *  sourceNode.connect(nextLoggerNode).connect(destinationNode);
 *
 **/

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LoggerNode = function () {
    function LoggerNode() {
        _classCallCheck(this, LoggerNode);

        this.connections = [];
        this.name = this.displayName || this.constructor.name || "<dummy>";
        // runtime information
        this.parentNode = null;
    }

    /**
     * LoggerNode main process
     * Should be override by sub classes implementation
     * @param {*} chunk chunk data is any type
     * @param {function(*)} next call the `next(chunk)` function after finish the process
     */


    _createClass(LoggerNode, [{
        key: "process",
        value: function process(chunk, next) {
            throw new Error("Should be implemented!\nclass YourNode extends LoggerNode{\n    process(chunk, next){\n        // process\n        next(chunk);\n    }\n}\n");
        }
    }, {
        key: "processIfNecessary",
        value: function processIfNecessary(currentChunk, parentNode) {
            var _this = this;

            // TODO: prevent infinite loop when audio graph has feedback
            this.parentNode = parentNode;
            this.process(currentChunk, function (nextChunk) {
                _this.parentNode = null;
                _this.outputs.forEach(function (outputNode) {
                    outputNode.processIfNecessary(nextChunk, _this);
                });
            });
        }

        /**
         * Get an array of all the nodes which connect to this node.
         *
         * @return {LoggerNode[]} An array of nodes which connect to this node.
         */

    }, {
        key: "connect",
        value: function connect(destinationNode) {
            this.connections.push({
                type: "name",
                source: this,
                destination: destinationNode
            });
            return destinationNode;
        }
    }, {
        key: "disconnect",
        value: function disconnect(sourceNode, destinationNode) {
            var _this2 = this;

            var toRemove = [];
            this.connections.forEach(function (connection) {
                if (connection.source === sourceNode && connection.destination === destinationNode) {
                    toRemove.push(connection);
                }
            });
            if (toRemove.length === 0) {
                return false;
            }
            toRemove.forEach(function (removeNode) {
                var index = _this2.connections.indexOf(removeNode);
                _this2.connections.splice(index, 1);
            });
            return true;
        }
    }, {
        key: "inputs",
        get: function get() {
            var _this3 = this;

            return this.connections.filter(function (connection) {
                return connection.destination === _this3 && connection.type === "name";
            }).map(function (connection) {
                return connection.source;
            });
        }

        /**
         * Get an array of all the nodes which this node outputs to.
         *
         * @return {LoggerNode[]} An array of nodes which this node connects to.
         */

    }, {
        key: "outputs",
        get: function get() {
            var _this4 = this;

            return this.connections.filter(function (connection) {
                return connection.source === _this4 && connection.type === "name";
            }).map(function (connection) {
                return connection.destination;
            });
        }
    }]);

    return LoggerNode;
}();

exports.default = LoggerNode;
//# sourceMappingURL=LoggerNode.js.map