// LICENSE : MIT
"use strict";
// https://github.com/Rich-Harris/estree-walker

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.walk = walk;
function walk(ast, _ref) {
    var enter = _ref.enter;
    var leave = _ref.leave;

    visit(ast, null, enter, leave);
}
var context = {
    skip: function skip() {
        return context.shouldSkip = true;
    }
};
var childKeys = {};
var toString = Object.prototype.toString;
function isArray(thing) {
    return toString.call(thing) === '[object Array]';
}

function visit(node, parent, enter, leave, prop, index) {
    if (!node) {
        return;
    }

    if (enter) {
        context.shouldSkip = false;
        enter.call(context, node, parent, prop, index);
        if (context.shouldSkip) {
            return;
        }
    }

    var keys = childKeys[node.type] || (childKeys[node.type] = Object.keys(node).filter(function (key) {
        return _typeof(node[key]) === 'object';
    }));

    var key = void 0,
        value = void 0,
        i = void 0,
        j = void 0;

    i = keys.length;
    while (i--) {
        key = keys[i];
        value = node[key];

        if (isArray(value)) {
            j = value.length;
            while (j--) {
                visit(value[j], node, enter, leave, key, j);
            }
        } else if (value && value.type) {
            visit(value, node, enter, leave, key, null);
        }
    }

    if (leave) {
        leave(node, parent, prop, index);
    }
}
//# sourceMappingURL=walker.js.map